"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@anycli/command");
const cli_ux_1 = require("cli-ux");
const engine_1 = require("./engine");
class Command extends command_1.default {
    constructor() {
        super(...arguments);
        this.engine = new engine_1.default();
    }
    async run() {
        await this.engine.load(this.config);
        let id = this.argv[0];
        if (id === '-h' || this.argv.includes('--help'))
            return this.showHelp();
        if (!id)
            id = 'help';
        await this.engine.runHook('init', { id });
        const cachedCommand = this.engine.findCommand(id);
        if (!cachedCommand)
            return this.commandNotFound(id);
        this.debug('found command', cachedCommand.id);
        const command = await cachedCommand.load();
        command.plugin = this.engine.plugins.find(p => p.name === cachedCommand.pluginName);
        this.debug('loaded command', command.id);
        await command.run(this.argv.slice(1), { config: this.config });
    }
    async commandNotFound(id) {
        await this.engine.runHook('command_not_found', { id });
        throw new Error(`command not found: ${id}`);
    }
    showHelp() {
        let id = this.argv.find(a => !a.startsWith('-'));
        const HHelp = require('@anycli/plugin-help').default;
        let help = new HHelp(this.config);
        if (!id) {
            let rootHelp = help.root();
            cli_ux_1.default.info(rootHelp);
        }
        else {
            let command = this.engine.findCommand(id, true);
            let commandHelp = help.command(command);
            cli_ux_1.default.info(commandHelp);
        }
    }
}
Command.type = 'engine';
exports.default = Command;
